@@ -37,7 +37,6 @@
 #include <gconf/gconf-client.h>
 
 #include <libgnomeui/gnome-ui-init.h>
-#include <libgnomeui/gnome-help.h> /* for gnome_help_display */
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -59,6 +58,8 @@
 #define KEY_CYCLE_DELAY     KEY_DIR "/cycle_delay"
 #define KEY_THEMES          KEY_DIR "/themes"
 
+#define GPM_COMMAND "gnome-power-preferences"
+
 enum {
         NAME_COLUMN = 0,
         ID_COLUMN,
@@ -404,24 +405,65 @@ preview_set_theme (GtkWidget  *widget,
         }
 }
 
+static void
+help_display (void)
+{
+	GError     *error = NULL;
+	char       *command;
+	const char *lang;
+	char       *uri = NULL;
+	GdkScreen  *gscreen;
+	int         i;
+
+	const char * const * langs = g_get_language_names ();
+
+	for (i = 0; langs[i] != NULL; i++) {
+		lang = langs[i];
+		if (strchr (lang, '.')) {
+			continue;
+		}
+
+		uri = g_build_filename (DATADIR,
+                                        "/gnome/help/user-guide/",
+					lang,
+                                        "/user-guide.xml",
+					NULL);
+
+		if (g_file_test (uri, G_FILE_TEST_EXISTS)) {
+                    break;
+		}
+	}
+
+	command = g_strconcat ("gnome-open ghelp://",
+                               uri,
+                               "?prefs-screensaver",
+                               NULL);
+	gscreen = gdk_screen_get_default ();
+	gdk_spawn_command_line_on_screen (gscreen, command, &error);
+
+	if (error != NULL) {
+		GtkWidget *d;
+
+		d = gtk_message_dialog_new (NULL,
+                                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                            GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+                                            "%s", error->message);
+		gtk_dialog_run (GTK_DIALOG (d));
+		gtk_widget_destroy (d);
+		g_error_free (error);
+	}
+
+	g_free (command);
+	g_free (uri);
+}
+
 static void
 response_cb (GtkWidget *widget,
              int        response_id)
 {
 
         if (response_id == GTK_RESPONSE_HELP) {
-                GError *error;
-                error = NULL;
-                gnome_help_display_desktop (NULL,
-                                            "user-guide",
-                                            "user-guide.xml",
-                                            "prefs-screensaver",
-                                            &error);
-                if (error != NULL) {
-                        g_warning ("%s", error->message);
-                        g_error_free (error);
-                }
-
+                help_display ();
         } else if (response_id == GTK_RESPONSE_REJECT) {
                 GError  *error;
                 gboolean res;
@@ -429,7 +471,7 @@ response_cb (GtkWidget *widget,
                 error = NULL;
 
                 res = gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
-                                                        "gnome-power-preferences",
+                                                        GPM_COMMAND,
                                                         &error);
                 if (! res) {
                         g_warning ("Unable to start power management preferences: %s", error->message);
@@ -1196,12 +1238,13 @@ fullscreen_preview_start_cb (GtkWidget *widget,
         gtk_widget_hide (dialog);
 
         fullscreen_preview_window = glade_xml_get_widget (xml, "fullscreen_preview_window");
-        gtk_widget_show (fullscreen_preview_window);
-        gtk_widget_grab_focus (fullscreen_preview_window);
 
         gtk_window_fullscreen (GTK_WINDOW (fullscreen_preview_window));
         gtk_window_set_keep_above (GTK_WINDOW (fullscreen_preview_window), TRUE);
 
+        gtk_widget_show (fullscreen_preview_window);
+        gtk_widget_grab_focus (fullscreen_preview_window);
+
         fullscreen_preview_area = glade_xml_get_widget (xml, "fullscreen_preview_area");
         gs_job_set_widget (job, fullscreen_preview_area);
 }
@@ -1300,8 +1343,8 @@ get_best_visual (void)
                         visual_id = (VisualID) v;
                         visual = gdkx_visual_get (visual_id);
 
-                        g_message ("Found best visual for GL: 0x%x",
-                                   (unsigned int) visual_id);
+                        g_debug ("Found best visual for GL: 0x%x",
+                                 (unsigned int) visual_id);
                 }
         }
 
@@ -1357,6 +1400,18 @@ setup_treeview_idle (gpointer data)
         return FALSE;
 }
 
+static gboolean
+is_program_in_path (const char *program)
+{
+        char *tmp = g_find_program_in_path (program);
+        if (tmp != NULL) {
+                g_free (tmp);
+                return TRUE;
+        } else {
+                return FALSE;
+        }
+}
+
 static void
 init_capplet (void)
 {
@@ -1371,6 +1426,7 @@ init_capplet (void)
         GtkWidget *lock_checkbox;
         GtkWidget *root_warning_label;
         GtkWidget *preview_button;
+        GtkWidget *gpm_button;
         GtkWidget *fullscreen_preview_window;
         GtkWidget *fullscreen_preview_previous;
         GtkWidget *fullscreen_preview_next;
@@ -1413,6 +1469,7 @@ init_capplet (void)
         lock_checkbox      = glade_xml_get_widget (xml, "lock_checkbox");
         root_warning_label = glade_xml_get_widget (xml, "root_warning_label");
         preview_button     = glade_xml_get_widget (xml, "preview_button");
+        gpm_button         = glade_xml_get_widget (xml, "gpm_button");
         fullscreen_preview_window = glade_xml_get_widget (xml, "fullscreen_preview_window");
         fullscreen_preview_area = glade_xml_get_widget (xml, "fullscreen_preview_area");
         fullscreen_preview_close = glade_xml_get_widget (xml, "fullscreen_preview_close");
@@ -1427,6 +1484,11 @@ init_capplet (void)
         gtk_widget_set_no_show_all (root_warning_label, TRUE);
         widget_set_best_colormap (preview);
 
+        if (! is_program_in_path (GPM_COMMAND)) {
+                gtk_widget_set_no_show_all (gpm_button, TRUE);
+                gtk_widget_hide (gpm_button);
+        }
+
         activate_delay = config_get_activate_delay (&is_writable);
         ui_set_delay (activate_delay);
         if (! is_writable) {
