@@ -39,6 +39,7 @@ static void gs_prefs_finalize   (GObject      *object);
 #define KEY_LOCK_ENABLED   KEY_DIR "/lock_enabled"
 #define KEY_MODE           KEY_DIR "/mode"
 #define KEY_ACTIVATE_DELAY KEY_DIR "/idle_delay"
+#define KEY_POWER_DELAY    KEY_DIR "/power_management_delay"
 #define KEY_LOCK_DELAY     KEY_DIR "/lock_delay"
 #define KEY_CYCLE_DELAY    KEY_DIR "/cycle_delay"
 #define KEY_THEMES         KEY_DIR "/themes"
@@ -46,6 +47,9 @@ static void gs_prefs_finalize   (GObject      *object);
 #define KEY_LOGOUT_ENABLED KEY_DIR "/logout_enabled"
 #define KEY_LOGOUT_DELAY   KEY_DIR "/logout_delay"
 #define KEY_LOGOUT_COMMAND KEY_DIR "/logout_command"
+#define KEY_KEYBOARD_ENABLED KEY_DIR "/embedded_keyboard_enabled"
+#define KEY_KEYBOARD_COMMAND KEY_DIR "/embedded_keyboard_command"
+#define KEY_AWAY_MESSAGE   KEY_DIR "/away_message"
 
 #define GS_PREFS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GS_TYPE_PREFS, GSPrefsPrivate))
 
@@ -70,7 +74,6 @@ static GConfEnumStringPair mode_enum_map [] = {
        { 0, NULL }
 };
 
-static GObjectClass *parent_class = NULL;
 static guint         signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GSPrefs, gs_prefs, G_TYPE_OBJECT)
@@ -106,8 +109,6 @@ gs_prefs_class_init (GSPrefsClass *klass)
 {
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
-
         object_class->finalize     = gs_prefs_finalize;
         object_class->get_property = gs_prefs_get_property;
         object_class->set_property = gs_prefs_set_property;
@@ -143,6 +144,22 @@ _gs_prefs_set_timeout (GSPrefs *prefs,
         prefs->timeout = value * 60000;
 }
 
+static void
+_gs_prefs_set_power_timeout (GSPrefs *prefs,
+                             int      value)
+{
+        if (value < 1)
+                value = 60;
+
+        /* pick a reasonable large number for the
+           upper bound */
+        if (value > 480)
+                value = 480;
+
+        /* this value is in seconds - others are in minutes */
+        prefs->power_timeout = value * 1000;
+}
+
 static void
 _gs_prefs_set_lock_timeout (GSPrefs *prefs,
                             int      value)
@@ -212,6 +229,35 @@ _gs_prefs_set_lock_enabled (GSPrefs *prefs,
         prefs->lock_enabled = value;
 }
 
+static void
+_gs_prefs_set_keyboard_enabled (GSPrefs *prefs,
+                                gboolean value)
+{
+        prefs->keyboard_enabled = value;
+}
+
+static void
+_gs_prefs_set_keyboard_command (GSPrefs    *prefs,
+                                const char *value)
+{
+        g_free (prefs->keyboard_command);
+        prefs->keyboard_command = NULL;
+
+       if (value) {
+               /* FIXME: check command */
+
+               prefs->keyboard_command = g_strdup (value);
+        }
+}
+
+static void
+_gs_prefs_set_away_message (GSPrefs    *prefs,
+                            const char *value)
+{
+        g_free (prefs->away_message);
+        prefs->away_message = g_strdup (value);
+}
+
 static void
 _gs_prefs_set_logout_enabled (GSPrefs *prefs,
                               gboolean value)
@@ -296,6 +342,13 @@ gs_prefs_load_from_gconf (GSPrefs *prefs)
                 key_error_and_free (KEY_ACTIVATE_DELAY, error);
         }
 
+        value = gconf_client_get_int (prefs->priv->gconf_client, KEY_POWER_DELAY, &error);
+        if (! error) {
+                _gs_prefs_set_power_timeout (prefs, value);
+        } else {
+                key_error_and_free (KEY_POWER_DELAY, error);
+        }
+
         value = gconf_client_get_int (prefs->priv->gconf_client, KEY_LOCK_DELAY, &error);
         if (! error) {
                 _gs_prefs_set_lock_timeout (prefs, value);
@@ -326,6 +379,31 @@ gs_prefs_load_from_gconf (GSPrefs *prefs)
                 key_error_and_free (KEY_THEMES, error);
         }
 
+        /* Embedded keyboard options */
+
+        bvalue = gconf_client_get_bool (prefs->priv->gconf_client, KEY_KEYBOARD_ENABLED, &error);
+        if (! error) {
+                _gs_prefs_set_keyboard_enabled (prefs, bvalue);
+        } else {
+                key_error_and_free (KEY_KEYBOARD_ENABLED, error);
+        }
+
+        string = gconf_client_get_string (prefs->priv->gconf_client, KEY_KEYBOARD_COMMAND, &error);
+        if (! error) {
+                _gs_prefs_set_keyboard_command (prefs, string);
+        } else {
+                key_error_and_free (KEY_KEYBOARD_COMMAND, error);
+        }
+        g_free (string);
+
+        string = gconf_client_get_string (prefs->priv->gconf_client, KEY_AWAY_MESSAGE, &error);
+        if (! error) {
+                _gs_prefs_set_away_message (prefs, string);
+        } else {
+                key_error_and_free (KEY_AWAY_MESSAGE, error);
+        }
+        g_free (string);
+
         /* Logout options */
 
         bvalue = gconf_client_get_bool (prefs->priv->gconf_client, KEY_LOGOUT_ENABLED, &error);
@@ -444,6 +522,19 @@ key_changed_cb (GConfClient *client,
                         invalid_type_warning (key);
                 }
 
+        } else if (strcmp (key, KEY_POWER_DELAY) == 0) {
+
+                if (value->type == GCONF_VALUE_INT) {
+                        int delay;
+
+                        delay = gconf_value_get_int (value);
+                        _gs_prefs_set_power_timeout (prefs, delay);
+
+                        changed = TRUE;
+                } else {
+                        invalid_type_warning (key);
+                }
+
         } else if (strcmp (key, KEY_LOCK_DELAY) == 0) {
 
                 if (value->type == GCONF_VALUE_INT) {
@@ -496,6 +587,45 @@ key_changed_cb (GConfClient *client,
                         invalid_type_warning (key);
                 }
 
+        } else if (strcmp (key, KEY_KEYBOARD_ENABLED) == 0) {
+
+                if (value->type == GCONF_VALUE_BOOL) {
+                        gboolean enabled;
+
+                        enabled = gconf_value_get_bool (value);
+                        _gs_prefs_set_keyboard_enabled (prefs, enabled);
+
+                        changed = TRUE;
+                } else {
+                        invalid_type_warning (key);
+                }
+
+        } else if (strcmp (key, KEY_KEYBOARD_COMMAND) == 0) {
+
+                if (value->type == GCONF_VALUE_STRING) {
+                        const char *command;
+
+                        command = gconf_value_get_string (value);
+                        _gs_prefs_set_keyboard_command (prefs, command);
+
+                        changed = TRUE;
+                } else {
+                        invalid_type_warning (key);
+                }
+
+        } else if (strcmp (key, KEY_AWAY_MESSAGE) == 0) {
+
+                if (value->type == GCONF_VALUE_STRING) {
+                        const char *away_message;
+
+                        away_message = gconf_value_get_string (value);
+                        _gs_prefs_set_away_message (prefs, away_message);
+
+                        changed = TRUE;
+                } else {
+                        invalid_type_warning (key);
+                }
+
         } else if (strcmp (key, KEY_LOGOUT_ENABLED) == 0) {
 
                 if (value->type == GCONF_VALUE_BOOL) {
@@ -564,20 +694,17 @@ gs_prefs_init (GSPrefs *prefs)
 
         prefs->priv->gconf_client      = gconf_client_get_default ();
 
-        prefs->verbose                 = FALSE;
-        prefs->debug                   = FALSE;
-
         prefs->idle_activation_enabled = TRUE;
         prefs->lock_enabled            = TRUE;
         prefs->logout_enabled          = FALSE;
         prefs->user_switch_enabled     = FALSE;
 
         prefs->timeout                 = 600000;
+        prefs->power_timeout           = 60000;
         prefs->lock_timeout            = 0;
         prefs->logout_timeout          = 14400000;
         prefs->cycle                   = 600000;
 
-
         prefs->mode                    = GS_MODE_SINGLE;
 
         /* GConf setup */
@@ -617,8 +744,10 @@ gs_prefs_finalize (GObject *object)
         }
 
         g_free (prefs->logout_command);
+        g_free (prefs->keyboard_command);
+        g_free (prefs->away_message);
 
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (gs_prefs_parent_class)->finalize (object);
 }
 
 GSPrefs *
