@@ -292,14 +292,41 @@ gs_window_clear (GSWindow *window)
         gdk_flush ();
 }
 
+static GdkRegion *
+get_outside_region (GSWindow *window)
+{
+        int i;
+        GdkRegion *region;
+
+        region = gdk_region_new ();
+        for (i = 0; i < window->priv->monitor; i++) {
+                GdkRectangle geometry;
+
+                gdk_screen_get_monitor_geometry (GTK_WINDOW (window)->screen,
+                                                   i, &geometry);
+                gdk_region_union_with_rect (region, &geometry);
+        }
+
+        return region;
+}
+
 static void
 update_geometry (GSWindow *window)
 {
         GdkRectangle geometry;
+        GdkRegion *outside_region, *monitor_region;
+
+        outside_region = get_outside_region (window);
 
         gdk_screen_get_monitor_geometry (GTK_WINDOW (window)->screen,
                                          window->priv->monitor,
                                          &geometry);
+        monitor_region = gdk_region_rectangle (&geometry);
+        gdk_region_subtract (monitor_region, outside_region);
+        gdk_region_destroy (outside_region); 
+
+        gdk_region_get_clipbox (monitor_region, &geometry);
+        gdk_region_destroy (monitor_region); 
 
         window->priv->geometry.x = geometry.x;
         window->priv->geometry.y = geometry.y;
@@ -794,72 +821,6 @@ gs_window_get_gdk_window (GSWindow *window)
         return GTK_WIDGET (window)->window;
 }
 
-static GPtrArray *
-get_env_vars (GtkWidget *widget)
-{
-        GPtrArray *env;
-        char      *str;
-        int        i;
-        static const char *allowed_env_vars [] = {
-                "PATH",
-                "LD_LIBRARY_PATH",
-                "SESSION_MANAGER",
-                "XAUTHORITY",
-                "XAUTHLOCALHOSTNAME",
-                "KRB5CCNAME",
-                "KRBTKFILE",
-                "LANG",
-                "LANGUAGE",
-                "LC_ALL",
-                "LC_CTYPE",
-                "LC_MESSAGES",
-                "LOGNAME",
-                "RUNNING_UNDER_GDM",
-                "GTK_DATA_PREFIX",
-                "GTK_EXE_PREFIX",
-                "GTK_IM_MODULE",
-                "GTK_IM_MODULE_FILE",
-                "GTK_MODULES",
-                "GTK_PATH",
-                "GTK2_RC_FILES",
-                "XDG_CONFIG_DIRS",
-                "XDG_DATA_DIRS",
-                "XDG_CACHE_HOME",
-                "XDG_CONFIG_HOME",
-                "XDG_DATA_HOME",
-                "MB_KBD_CONFIG",
-                "MB_KBD_VARIANT",
-                "MB_KBD_LANG",
-                "DBUS_SESSION_BUS_ADDRESS"
-        };
-
-        env = g_ptr_array_new ();
-
-        str = gdk_screen_make_display_name (gtk_widget_get_screen (widget));
-        g_ptr_array_add (env, g_strdup_printf ("DISPLAY=%s", str));
-        g_free (str);
-
-        g_ptr_array_add (env, g_strdup_printf ("HOME=%s",
-                                               g_get_home_dir ()));
-
-        for (i = 0; i < G_N_ELEMENTS (allowed_env_vars); i++) {
-                const char *var;
-                const char *val;
-                var = allowed_env_vars [i];
-                val = g_getenv (var);
-                if (val != NULL) {
-                        char *str;
-                        str = g_strdup_printf ("%s=%s", var, val);
-                        g_ptr_array_add (env, str);
-                        gs_debug ("adding environment: %s", str);
-                }
-        }
-
-        g_ptr_array_add (env, NULL);
-
-        return env;
-}
-
 /* just for debugging */
 static gboolean
 error_watch (GIOChannel   *source,
@@ -913,7 +874,6 @@ spawn_on_window (GSWindow *window,
 {
         int         argc;
         char      **argv;
-        GPtrArray  *env;
         GError     *error;
         gboolean    result;
         GIOChannel *channel;
@@ -930,13 +890,11 @@ spawn_on_window (GSWindow *window,
                 return FALSE;
         }
 
-        env = get_env_vars (GTK_WIDGET (window));
-
         error = NULL;
         result = gdk_spawn_on_screen_with_pipes (GTK_WINDOW (window)->screen,
                                                  NULL,
                                                  argv,
-                                                 (char **)env->pdata,
+                                                 NULL,
                                                  G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
                                                  NULL,
                                                  NULL,
@@ -946,11 +904,6 @@ spawn_on_window (GSWindow *window,
                                                  &standard_error,
                                                  &error);
 
-        for (i = 0; i < env->len; i++) {
-                g_free (g_ptr_array_index (env, i));
-        }
-        g_ptr_array_free (env, TRUE);
-
         if (! result) {
                 gs_debug ("Could not start command '%s': %s", command, error->message);
                 g_error_free (error);
@@ -1467,8 +1420,8 @@ popup_dialog_idle (GSWindow *window)
         }
 
 	if (window->priv->away_message) {
-		gchar *quoted;
-		
+		char *quoted;
+
 		quoted = g_shell_quote (window->priv->away_message);
 		g_string_append_printf (command, " --away-message=%s", quoted);
 		g_free (quoted);
